package com.sun.jmx.snmp.tasks;

import java.util.ArrayList;

public class ThreadService implements TaskServer
{
    private ArrayList<Runnable> jobList;
    private ThreadService.ExecutorThread[] threadList;
    private int minThreads;
    private int currThreds;
    private int idle;
    private boolean terminated;
    private int priority;
    private ThreadGroup threadGroup;
    private ClassLoader cloader;
    private static long counter;
    private int addedJobs;
    private int doneJobs;
    
    public ThreadService(int minThreads) {
        this.jobList = new ArrayList(0);
        this.minThreads = 1;
        this.currThreds = 0;
        this.idle = 0;
        this.terminated = false;
        this.threadGroup = new ThreadGroup("ThreadService");
        this.addedJobs = 1;
        this.doneJobs = 1;
        if (minThreads <= 0) {
            throw new IllegalArgumentException("The thread number should bigger than zero.");
        }
        this.minThreads = minThreads;
        this.threadList = new ThreadService.ExecutorThread[minThreads];
        this.priority = Thread.currentThread().getPriority();
        this.cloader = Thread.currentThread().getContextClassLoader();
        return;
    }
    
    @Override
    public void submitTask(Task task) throws IllegalArgumentException {
        this.submitTask(task);
    }
    
    public void submitTask(Runnable element) throws IllegalArgumentException {
        this.stateCheck();
        if (element == null) {
            throw new IllegalArgumentException("No task specified.");
        }
        synchronized (this.jobList) {
            this.jobList.add(this.jobList.size(), element);
            this.jobList.notify();
        }
        this.createThread();
        return;
    }
    
    public Runnable removeTask(Runnable o) {
        this.stateCheck();
        Runnable runnable = null;
        synchronized (this.jobList) {
            int index = this.jobList.indexOf(o);
            if (index >= 0) {
                runnable = (Runnable)this.jobList.remove(index);
            }
        }
        if (runnable != null && runnable instanceof Task) {
            ((Task)runnable).cancel();
            return runnable;
        }
        return runnable;
    }
    
    public void removeAll() {
        this.stateCheck();
        Object[] array;
        synchronized (this.jobList) {
            array = this.jobList.toArray();
            this.jobList.clear();
        }
        for (Object o : array) {
            if (o != null && o instanceof Task) {
                ((Task)o).cancel();
            }
        }
    }
    
    public void terminate() {
        if (this.terminated) {
            return;
        }
        this.terminated = true;
        synchronized (this.jobList) {
            this.jobList.notifyAll();
        }
        this.removeAll();
        for (int i = 0; i < this.currThreds; ++i) {
            try {
                this.threadList[i].interrupt();
            }
            catch (Exception ex) {}
        }
        this.threadList = null;
        return;
    }
    
    private void stateCheck() throws IllegalStateException {
        if (this.terminated) {
            throw new IllegalStateException("The thread service has been terminated.");
        }
        return;
    }
    
    private void createThread() {
        if (this.idle < 1) {
            synchronized (this.threadList) {
                if (this.jobList.size() > 0 && this.currThreds < this.minThreads) {
                    ThreadService.ExecutorThread executorThread = new ThreadService.ExecutorThread();
                    executorThread.start();
                    this.threadList[this.currThreds++] = executorThread;
                }
            }
        }
    }
    
    static {
        ThreadService.counter = 0L;
    }
    
    private class ExecutorThread extends Thread
    {
        public ExecutorThread() {
            super(ThreadService.this.threadGroup, "ThreadService-" + ThreadService.counter++);
            this.setDaemon(true);
            this.setPriority(ThreadService.this.priority);
            this.setContextClassLoader(ThreadService.this.cloader);
            ThreadService.this.idle++;
        }
        
        @Override
        public void run() {
            while (!ThreadService.this.terminated) {
                Runnable runnable = null;
                synchronized (ThreadService.this.jobList) {
                    if (ThreadService.this.jobList.size() > 0) {
                        runnable = (Runnable)ThreadService.this.jobList.remove(0);
                        if (ThreadService.this.jobList.size() > 0) {
                            ThreadService.this.jobList.notify();
                        }
                    }
                    else {
                        try {
                            ThreadService.this.jobList.wait();
                        }
                        catch (InterruptedException ex) {}
                        continue;
                    }
                }
                if (runnable != null) {
                    try {
                        ThreadService.this.idle--;
                        runnable.run();
                    }
                    catch (Exception ex2) {
                        ex2.printStackTrace();
                    }
                    finally {
                        ThreadService.this.idle++;
                    }
                }
                this.setPriority(ThreadService.this.priority);
                Thread.interrupted();
                this.setContextClassLoader(ThreadService.this.cloader);
            }
        }
    }
}
